import logging
from tortoise import run_async
from tenacity import retry, after_log, before_log, retry, wait_fixed, stop_after_attempt
from tortoise import Tortoise

try:
    import app
except ModuleNotFoundError:
    import sys
    import os
    import pathlib

    app = pathlib.Path(os.path.dirname(__file__)).parent
    sys.path.append(str(app))
    from app import crud, schemas
    from app.core.config import settings
    from app.core.config import TORTOISE_ORM

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


@retry(
    wait=wait_fixed(1),
    stop=stop_after_attempt(30),
    before=before_log(logger, logging.INFO),
    after=after_log(logger, logging.WARN),
)
async def initial():
    try:
        await Tortoise.init(config=TORTOISE_ORM)
        user = await crud.user.get_by_email(email=settings.FIRST_SUPERUSER_EMAIL)
        if not user:
            user_data = schemas.UserCreateBySuperuser(
                email=settings.FIRST_SUPERUSER_EMAIL,
                password=settings.FIRST_SUPERUSER_PASSWORD,
                is_superuser=True,
            )
            await crud.user.create_by_superuser(user_data)
    except Exception as e:
        logger.error(e)
        raise Exception(e)


async def main() -> None:
    logger.info("Creating initial data")
    await initial()
    logger.info("Initial data created")


if __name__ == "__main__":
    run_async(main())



from typing import Optional

from app.core.security import get_password_hash, verify_password
from app.crud.base import CRUDBase
from app import models, schemas
from tortoise.exceptions import DoesNotExist


class CRUDUser(CRUDBase[models.User, schemas.UserCreateMe, schemas.UserUpdateMe]):
    async def get_by_email(self, email: str) -> Optional[models.User]:
        try:
            user = await models.User.get(email=email)
        except DoesNotExist:
            return None
        else:
            return user

    async def create_me(self, obj_in: schemas.UserCreateMe) -> models.User:
        db_obj = await models.User.create(
            email=obj_in.email,
            password_hash=get_password_hash(obj_in.password),
        )
        return db_obj

    async def create_by_superuser(
        self, obj_in: schemas.UserCreateBySuperuser
    ) -> models.User:

        db_obj = await models.User.create(
            email=obj_in.email,
            password_hash=get_password_hash(obj_in.password),
            is_superuser=obj_in.is_superuser,
            is_active=obj_in.is_active,
        )
        return db_obj

    async def update_me(
        self, db_obj: models.User, obj_in: schemas.UserUpdateMe
    ) -> models.User:
        db_obj.name = obj_in.name
        db_obj.family_name = obj_in.family_name
        if obj_in.password:
            new_password = get_password_hash(obj_in.password)
            db_obj.password_hash = new_password
        await db_obj.save()
        await db_obj.refresh_from_db()
        return db_obj

    async def update_by_superuser(
        self, db_obj: models.User, obj_in: schemas.UserUpdateBySuperuser
    ) -> models.User:

        db_obj.name = obj_in.name
        db_obj.family_name = obj_in.family_name
        if obj_in.password:
            new_password = get_password_hash(obj_in.password)
            db_obj.password_hash = new_password
        db_obj.is_superuser = obj_in.is_superuser  # type: ignore

        db_obj.is_active = obj_in.is_active  # type: ignore
        await db_obj.save()
        await db_obj.refresh_from_db()
        return db_obj

    async def authenticate(self, email: str, password: str) -> Optional[models.User]:
        user = await self.get_by_email(email=email)
        if not user:
            return None
        if not verify_password(password, user.password_hash):
            return None
        return user

    def is_active(self, user: models.User) -> bool:
        return user.is_active  # type: ignore

    def is_superuser(self, user: models.User) -> bool:
        return user.is_superuser  # type: ignore


user = CRUDUser(models.User)



import logging
from pathlib import Path

from cookiecutter.exceptions import OutputDirExistsException
from cookiecutter.main import cookiecutter

TEMPLATE_DIR = Path(__file__).parent

logging.basicConfig(level=logging.INFO, format="%(levelname)s:%(name)s -   %(message)s")

logger = logging.getLogger(
    name="fastapi-plan",
)

# print() below for newlines


def main():
    try:
        print()
        logger.warning("Enter fiew basic information about project")
        print()
        cookiecutter(template=f"{TEMPLATE_DIR}/template")
    except OutputDirExistsException:
        print()
        logger.error("FAILED")
        logger.error("Directory with such a name already exists!")
    except Exception as exc:
        print()
        logger.error("FAILED")
        logger.error(f"{exc}")
    else:
        print()
        logger.info("SUCCESS")
        logger.info("Project successfully generated.")


if __name__ == "__main__":
    main()



from fastapi import APIRouter

from app.api.routers import login, users


api_router = APIRouter()
api_router.include_router(login.router, prefix="/login", tags=["login"])
api_router.include_router(users.router, prefix="/users", tags=["users"])



from typing import Dict, List, Optional, Union
from pydantic import AnyHttpUrl, BaseSettings, AnyUrl, validator, EmailStr
from pathlib import Path

PROJECT_DIR = Path(__file__).parent.parent.parent


class Settings(BaseSettings):
    # DEBUG
    DEBUG: bool
    ACCESS_TOKEN_EXPIRE_MINUTES: int

    # SECURITY
    SECRET_KEY: str

    # PROJECT NAME, API PREFIX, CORS ORIGINS
    PROJECT_NAME: str
    API_STR: str
    BACKEND_CORS_ORIGINS: Union[str, List[AnyHttpUrl]]

    # POSTGRESQL
    POSTGRES_USER: str
    POSTGRES_PASSWORD: str
    POSTGRES_SERVER: str
    POSTGRES_PORT: str
    POSTGRES_DB: str
    TORTOISE_DATABASE_URI: Optional[str] = None

    # FIRST SUPERUSER
    FIRST_SUPERUSER_EMAIL: EmailStr
    FIRST_SUPERUSER_PASSWORD: str

    # VALIDATORS
    @validator("BACKEND_CORS_ORIGINS", pre=True)
    def _assemble_cors_origins(cls, cors_origins):
        if isinstance(cors_origins, str):
            return [item.strip() for item in cors_origins.split(",")]
        return cors_origins

    @validator("TORTOISE_DATABASE_URI", pre=True)
    def _assemble_db_connection(cls, v: Optional[str], values: Dict[str, str]) -> str:
        if isinstance(v, str):
            return v
        if values.get("DEBUG"):
            postgres_server = "localhost"
        else:
            postgres_server = values.get("POSTGRES_SERVER")

        return AnyUrl.build(
            scheme="postgres",
            user=values.get("POSTGRES_USER"),
            password=values.get("POSTGRES_PASSWORD"),
            host=postgres_server or "localhost",
            port=values.get("POSTGRES_PORT"),
            path=f"/{values.get('POSTGRES_DB')}",
        )

    class Config:
        env_file = f"{PROJECT_DIR}/.env"
        case_sensitive = True


settings: Settings = Settings()


TORTOISE_ORM = {
    "connections": {"default": settings.TORTOISE_DATABASE_URI},
    "apps": {
        "models": {
            "models": ["app.models", "aerich.models"],
            "default_connection": "default",
        },
    },
}



from typing import Optional
from tortoise.contrib.pydantic.creator import (
    pydantic_model_creator,
    pydantic_queryset_creator,
)
from pydantic import BaseModel, EmailStr
from app.models import User

UserPydantic = pydantic_model_creator(User)
UserPydanticList = pydantic_queryset_creator(User)


class UserCreateMe(BaseModel):
    email: EmailStr
    password: str

    class Config:
        orm_mode = True


class UserUpdateMe(BaseModel):
    password: Optional[str]
    name: Optional[str]
    family_name: Optional[str]

    class Config:
        orm_mode = True


class UserCreateBySuperuser(BaseModel):
    email: EmailStr
    password: str
    is_superuser: Optional[bool] = False
    is_active: Optional[bool] = True


class UserUpdateBySuperuser(UserCreateBySuperuser):
    email: Optional[EmailStr]
    password: Optional[str]
    name: Optional[str]
    family_name: Optional[str]



from app.schemas import UserCreateMe, UserCreateBySuperuser
from app.tests.utils.utils import random_email, random_lower_string


def get_random_user_me() -> UserCreateMe:
    return UserCreateMe(
        email=random_email(),
        password=random_lower_string(),
    )


def get_random_user_by_superuser() -> UserCreateBySuperuser:
    return UserCreateBySuperuser(
        email=random_email(),
        password=random_lower_string(),
    )


from app.core.config import settings
import random
import string
from asyncio import AbstractEventLoop as EventLoop
from typing import Dict
from app import crud
from fastapi.testclient import TestClient
from app.schemas import UserCreateBySuperuser


def random_integer_below_100() -> int:
    return random.randint(0, 99)


def random_lower_string(length=20) -> str:
    return "".join(random.choices(string.ascii_lowercase, k=length))


def random_email(length=10) -> str:
    return f"{random_lower_string(length)}@{random_lower_string(length)}.com"


def user_authentication_headers(
    client: TestClient,
    event_loop: EventLoop,
    email: str,
    password: str,
    is_superuser: bool = False,
    is_active: bool = True,
) -> Dict[str, str]:

    data = {"username": email, "password": password}
    user_in = UserCreateBySuperuser(
        email=email, password=password, is_superuser=is_superuser, is_active=is_active
    )
    event_loop.run_until_complete(crud.user.create_by_superuser(user_in))
    r = client.post(f"{settings.API_STR}/login/access-token", data=data)
    response = r.json()
    auth_token = response["access_token"]
    headers = {"Authorization": f"Bearer {auth_token}"}
    return headers



from typing import Optional
import tortoise.fields.data as fields
from tortoise.models import Model


class User(Model):
    """
    The User model
    """

    id: int = fields.IntField(pk=True)
    email: str = fields.CharField(max_length=50, unique=True)
    name: Optional[str] = fields.CharField(max_length=50, null=True, default=None)
    family_name: Optional[str] = fields.CharField(
        max_length=50, null=True, default=None
    )
    password_hash: str = fields.CharField(max_length=128)
    created_at = fields.DatetimeField(auto_now_add=True)
    modified_at = fields.DatetimeField(auto_now=True)
    is_active = fields.BooleanField(default=True)
    is_superuser = fields.BooleanField(default=False)

    def full_name(self) -> str:
        """
        Returns the best name
        """
        if self.name or self.family_name:
            return f"{self.name or ''} {self.family_name or ''}".strip()
        return self.email

    class PydanticMeta:
        computed = ["full_name"]
        exclude = ["password_hash"]



import pytest
from asyncio import AbstractEventLoop as EventLoop
from typing import Generator
from fastapi.testclient import TestClient
from tortoise.contrib.test import finalizer, initializer
from app.main import create_app
from app import models, schemas, crud
from app.tests.utils.utils import user_authentication_headers

app = create_app()

default_superuser = schemas.UserCreateBySuperuser(
    email="admin@admin.com",
    password="admin",
    is_superuser=True,
    is_active=True,
)

default_user = schemas.UserCreateBySuperuser(
    email="user@user.com",
    password="user",
    is_superuser=False,
    is_active=True,
)


@pytest.fixture(scope="module")
def client() -> Generator:
    initializer(["app.models"])
    with TestClient(app) as c:
        yield c
    finalizer()


@pytest.fixture(scope="module")
def event_loop(client: TestClient) -> Generator:
    yield client.task.get_loop()


@pytest.fixture(scope="module")
def normal_user(event_loop: EventLoop) -> Generator:
    user: models.User = event_loop.run_until_complete(
        crud.user.create_by_superuser(default_user)
    )
    yield user
    event_loop.run_until_complete(user.delete())


@pytest.fixture(scope="module")
def superuser_token_headers(client: TestClient, event_loop: EventLoop) -> Generator:
    headers = user_authentication_headers(
        client=client,
        event_loop=event_loop,
        email=default_superuser.email,
        password=default_superuser.password,
        is_superuser=default_superuser.is_superuser or True,
        is_active=default_superuser.is_active or True,
    )
    yield headers


@pytest.fixture(scope="module")
def normal_user_token_headers(client: TestClient, event_loop: EventLoop) -> Generator:
    headers = user_authentication_headers(
        client=client,
        event_loop=event_loop,
        email=default_user.email,
        password=default_user.password,
        is_superuser=default_user.is_superuser or False,
        is_active=default_user.is_active or True,
    )
    yield headers



import pytest
from app.schemas.user import UserUpdateBySuperuser, UserUpdateMe
from asyncio import AbstractEventLoop as EventLoop
from fastapi.testclient import TestClient
from app.models import User
from app import crud
from app.schemas import UserCreateMe, UserCreateBySuperuser
from app.tests.utils.utils import random_email, random_lower_string
from app.core.security import verify_password
import logging


logger = logging.getLogger(__name__)


@pytest.fixture(autouse=True)
async def drop() -> None:
    yield
    await User.all().delete()


def test_user_create_me(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password = random_lower_string()
    user_in = UserCreateMe(email=email, password=password)
    user = event_loop.run_until_complete(crud.user.create_me(user_in))

    assert user.email == email
    assert hasattr(user, "password_hash")
    assert user.is_superuser == False


def test_user_create_by_superuser(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password = random_lower_string()
    is_superuser = True
    is_active = False
    user_in = UserCreateBySuperuser(
        email=email,
        password=password,
        is_superuser=is_superuser,
        is_active=is_active,
    )
    user = event_loop.run_until_complete(crud.user.create_by_superuser(user_in))
    assert user.email == email
    assert hasattr(user, "password_hash")
    assert user.is_superuser == is_superuser
    assert user.is_active == is_active


def test_user_authenticate(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password = random_lower_string()
    user_in = UserCreateMe(email=email, password=password)
    user = event_loop.run_until_complete(crud.user.create_me(user_in))
    assert verify_password(password, user.password_hash)


def test_user_update_me(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password_hash = random_lower_string()
    user = event_loop.run_until_complete(
        User.create(email=email, password_hash=password_hash)
    )

    name = random_lower_string()
    family_name = random_lower_string()
    new_password = random_lower_string()

    user_in = UserUpdateMe(name=name, family_name=family_name, password=new_password)
    updated_user = event_loop.run_until_complete(crud.user.update_me(user, user_in))
    assert updated_user.name == name
    assert updated_user.family_name == family_name
    assert verify_password(new_password, updated_user.password_hash)


def test_user_update_by_superuser(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password_hash = random_lower_string()
    user = event_loop.run_until_complete(
        User.create(email=email, password_hash=password_hash)
    )

    name = random_lower_string()
    family_name = random_lower_string()
    new_password = random_lower_string()
    new_is_superuser = True
    new_is_active = False

    user_in = UserUpdateBySuperuser(
        name=name,
        family_name=family_name,
        password=new_password,
        is_active=new_is_active,
        is_superuser=new_is_superuser,
    )
    updated_user = event_loop.run_until_complete(
        crud.user.update_by_superuser(user, user_in)
    )
    assert updated_user.name == name
    assert updated_user.family_name == family_name
    assert verify_password(new_password, updated_user.password_hash)
    assert updated_user.is_active == new_is_active
    assert updated_user.is_superuser == new_is_superuser


# Above 5 tests are slow (and should be), becouse of hash/verfiy inside takes a lot of time!


def test_user_is_active_not(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password_hash = random_lower_string()
    user = event_loop.run_until_complete(
        User.create(email=email, password_hash=password_hash)
    )
    assert crud.user.is_active(user) == True


def test_user_is_active_active(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password_hash = random_lower_string()
    user = event_loop.run_until_complete(
        User.create(email=email, password_hash=password_hash, is_active=False)
    )
    assert crud.user.is_active(user) == False


def test_user_is_superuser_not(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password_hash = random_lower_string()
    user = event_loop.run_until_complete(
        User.create(email=email, password_hash=password_hash, is_superuser=False)
    )
    assert crud.user.is_superuser(user) == False


def test_user_is_superuser_superuser(client: TestClient, event_loop: EventLoop):
    email = random_email()
    password_hash = random_lower_string()
    user = event_loop.run_until_complete(
        User.create(email=email, password_hash=password_hash, is_superuser=True)
    )
    assert crud.user.is_superuser(user) == True



from fastapi import APIRouter, Depends, HTTPException, status
from app import crud, models, schemas
from app.api import deps

router = APIRouter()


@router.get("/", response_model=schemas.UserPydanticList)
async def read_users(
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_superuser),
):
    """
    Retrieve users.
    """
    users = await crud.user.get_multi(skip=skip, limit=limit)
    return await schemas.UserPydanticList.from_queryset(users)


@router.post("/", response_model=schemas.UserPydantic)
async def create_user(
    user_in: schemas.UserCreateBySuperuser,
    current_user: models.User = Depends(deps.get_current_active_superuser),
):
    """
    Create new user.
    """
    user = await crud.user.get_by_email(email=user_in.email)
    if user:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="The user with this username already exists in the system.",
        )
    user = await crud.user.create_by_superuser(obj_in=user_in)
    return await schemas.UserPydantic.from_tortoise_orm(user)


@router.put("/me", response_model=schemas.UserPydantic)
async def update_user_me(
    user_in: schemas.UserUpdateMe,
    current_user: models.User = Depends(deps.get_current_active_user),
):
    """
    Update own user.
    """
    user = await crud.user.update_me(db_obj=current_user, obj_in=user_in)
    return await schemas.UserPydantic.from_tortoise_orm(user)


@router.get("/me", response_model=schemas.UserPydantic)
async def read_user_me(
    current_user: models.User = Depends(deps.get_current_active_user),
):
    """
    Get current user.
    """
    return await schemas.UserPydantic.from_tortoise_orm(current_user)


@router.post("/open", response_model=schemas.UserPydantic)
async def create_user_open(user_in: schemas.UserCreateMe):
    """
    Create new user without the need to be logged in.
    """
    user = await crud.user.get_by_email(email=user_in.email)
    if user:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="The user with this username already exists in the system",
        )
    user = await crud.user.create_me(obj_in=user_in)
    return await schemas.UserPydantic.from_tortoise_orm(user)


@router.get("/{user_id}", response_model=schemas.UserPydantic)
async def read_user_by_id(
    user_id: int,
    current_user: models.User = Depends(deps.get_current_active_user),
):
    """
    Get a specific user by id.
    """
    user = await crud.user.get(id=user_id)
    if user == current_user:
        return user
    if not user:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST, detail="The user does not exist"
        )
    if not crud.user.is_superuser(current_user):
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="The user doesn't have enough privileges",
        )
    return await schemas.UserPydantic.from_tortoise_orm(user)


@router.put("/{user_id}", response_model=schemas.UserPydantic)
async def update_user(
    user_id: int,
    user_in: schemas.UserUpdateBySuperuser,
    current_user: models.User = Depends(deps.get_current_active_superuser),
):
    """
    Update a user.
    """
    user = await crud.user.get(id=user_id)
    if not user:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="The user with this username does not exist in the system",
        )
    user = await crud.user.update_by_superuser(db_obj=user, obj_in=user_in)
    return await schemas.UserPydantic.from_tortoise_orm(user)



from typing import Any, Dict, Generic, Optional, Type, TypeVar, Union
from tortoise.models import Model
from fastapi.encoders import jsonable_encoder
from pydantic import BaseModel
from tortoise.queryset import QuerySet
from tortoise.exceptions import DoesNotExist

ModelType = TypeVar("ModelType", bound=Model)
CreateSchemaType = TypeVar("CreateSchemaType", bound=BaseModel)
UpdateSchemaType = TypeVar("UpdateSchemaType", bound=BaseModel)


class CRUDBase(Generic[ModelType, CreateSchemaType, UpdateSchemaType]):
    def __init__(self, model: Type[ModelType]):
        """
        CRUD object with default methods to Create, Read, Update, Delete (CRUD).
        **Parameters**
        * `model`: A Tortoise model class
        * `schema`: A Pydantic model (schema) class
        """
        self.model = model

    async def get(self, id: Any) -> Optional[ModelType]:
        try:
            instance = await self.model.get(id=id)
        except DoesNotExist:
            return None
        else:
            return instance

    async def get_multi(self, skip: int = 0, limit: int = 100) -> QuerySet[ModelType]:
        query = self.model.all().offset(skip).limit(limit)
        return query

    async def create(self, obj_in: CreateSchemaType) -> ModelType:
        obj_in_data = jsonable_encoder(obj_in)
        db_obj = await self.model.create(**obj_in_data)
        return db_obj

    async def update(
        self, db_obj: ModelType, obj_in: Union[UpdateSchemaType, Dict[str, Any]]
    ) -> ModelType:
        obj_data = jsonable_encoder(db_obj)
        if isinstance(obj_in, dict):
            update_data = obj_in
        else:
            update_data = obj_in.dict(exclude_unset=True)
        for field in obj_data:
            if field in update_data:
                setattr(db_obj, field, update_data[field])
        await db_obj.save()
        return db_obj

    async def remove(self, id: int) -> None:
        await self.model.filter(id=id).delete()



from fastapi import Depends, HTTPException, status
from fastapi.security import OAuth2PasswordBearer
from jose import jwt  # type: ignore
from pydantic import ValidationError

from app import crud, models, schemas
from app.core import security
from app.core.config import settings


reusable_oauth2 = OAuth2PasswordBearer(
    tokenUrl=f"{settings.API_STR}/login/access-token"
)


async def get_current_user(token: str = Depends(reusable_oauth2)) -> models.User:
    try:
        payload = jwt.decode(  # type: ignore
            token, settings.SECRET_KEY, algorithms=[security.ALGORITHM]
        )
        token_data = schemas.TokenPayload(**payload)
    except (jwt.JWTError, ValidationError):
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Could not validate credentials",
        )
    user = await crud.user.get(id=token_data.sub)
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    return user


async def get_current_active_user(
    current_user: models.User = Depends(get_current_user),
) -> models.User:
    if not crud.user.is_active(current_user):
        raise HTTPException(status_code=400, detail="Inactive user")
    return current_user


async def get_current_active_superuser(
    current_user: models.User = Depends(get_current_user),
) -> models.User:
    if not crud.user.is_superuser(current_user):
        raise HTTPException(
            status_code=400, detail="The user doesn't have enough privileges"
        )
    return current_user



from app import crud
from typing import Dict
from app.tests.conftest import default_user, default_superuser
from app.core.config import settings
from asyncio import AbstractEventLoop as EventLoop
from fastapi.testclient import TestClient
from app.tests.utils.utils import random_lower_string
from app.tests.utils.user import get_random_user_me, get_random_user_by_superuser


def test_read_users_superuser(
    client: TestClient,
    event_loop: EventLoop,
    normal_user_token_headers: Dict[str, str],
    superuser_token_headers: Dict[str, str],
) -> None:
    r = client.get(f"{settings.API_STR}/users/", headers=superuser_token_headers)
    users = r.json()
    assert len(users) >= 2
    for user in users:
        assert "email" in user
        assert "created_at" in user


def test_create_user_superuser(
    client: TestClient,
    event_loop: EventLoop,
    superuser_token_headers: Dict[str, str],
) -> None:
    data = get_random_user_by_superuser().dict()

    r = client.post(
        f"{settings.API_STR}/users/", headers=superuser_token_headers, json=data
    )

    current_user = r.json()
    assert r.status_code == 200
    assert current_user["email"] == data["email"]


def test_read_user_me_superuser(
    client: TestClient,
    event_loop: EventLoop,
    superuser_token_headers: Dict[str, str],
) -> None:
    r = client.get(f"{settings.API_STR}/users/me/", headers=superuser_token_headers)
    current_user = r.json()
    assert current_user
    assert current_user["is_active"] == default_superuser.is_active
    assert current_user["is_superuser"] == default_superuser.is_superuser
    assert current_user["email"] == default_superuser.email


def test_read_user_by_id_superuser(
    client: TestClient, event_loop: EventLoop, superuser_token_headers: Dict[str, str]
):
    normal_user = event_loop.run_until_complete(
        crud.user.get_by_email(default_user.email)
    )

    r = client.get(
        f"{settings.API_STR}/users/{normal_user.id}", headers=superuser_token_headers
    )
    assert r.status_code == 200
    current_user = r.json()
    assert current_user
    assert current_user["email"] == default_user.email


def test_update_user_by_id_superuser(
    client: TestClient, event_loop: EventLoop, superuser_token_headers: Dict[str, str]
):
    normal_user = event_loop.run_until_complete(
        crud.user.get_by_email(default_user.email)
    )

    data = {"is_active": False, "name": random_lower_string()}

    r = client.put(
        f"{settings.API_STR}/users/{normal_user.id}",
        headers=superuser_token_headers,
        json=data,
    )
    assert r.status_code == 200
    current_user = r.json()
    assert current_user
    assert current_user["is_active"] == data["is_active"]
    assert current_user["name"] == data["name"]
    ## cleaup to previous attributes
    event_loop.run_until_complete(normal_user.save())


def test_read_users_user(
    client: TestClient, event_loop: EventLoop, normal_user_token_headers: Dict[str, str]
) -> None:
    r = client.get(f"{settings.API_STR}/users/", headers=normal_user_token_headers)
    assert r.status_code == 400


def test_create_user_user(
    client: TestClient, event_loop: EventLoop, normal_user_token_headers: Dict[str, str]
) -> None:
    data = get_random_user_by_superuser().dict()
    r = client.post(
        f"{settings.API_STR}/users/", headers=normal_user_token_headers, json=data
    )
    assert r.status_code == 400


def test_read_user_me_user(
    client: TestClient, event_loop: EventLoop, normal_user_token_headers: Dict[str, str]
) -> None:
    r = client.get(f"{settings.API_STR}/users/me/", headers=normal_user_token_headers)
    current_user = r.json()
    assert current_user
    assert current_user["is_active"] == default_user.is_active
    assert current_user["is_superuser"] == default_user.is_superuser
    assert current_user["email"] == default_user.email


def test_update_user_me_user(
    client: TestClient, event_loop: EventLoop, normal_user_token_headers: Dict[str, str]
) -> None:
    name = random_lower_string()
    family_name = random_lower_string()
    data = {"name": name, "family_name": family_name}

    r = client.put(
        f"{settings.API_STR}/users/me", headers=normal_user_token_headers, json=data
    )
    assert r.status_code == 200
    current_user = r.json()
    assert current_user
    assert current_user["is_active"] == default_user.is_active
    assert current_user["name"] == name
    assert current_user["family_name"] == family_name
    assert current_user["is_superuser"] == default_user.is_superuser
    assert current_user["email"] == default_user.email


def test_create_user_open(
    client: TestClient, event_loop: EventLoop, normal_user_token_headers: Dict[str, str]
) -> None:
    data = get_random_user_me().dict()

    r = client.post(
        f"{settings.API_STR}/users/open", headers=normal_user_token_headers, json=data
    )

    assert r.status_code == 200
    current_user = r.json()
    assert current_user
    assert current_user["email"] == data["email"]
    assert current_user["name"] == None
    assert current_user["family_name"] == None
    assert current_user["is_active"] == True
    assert current_user["is_superuser"] == False

    created = event_loop.run_until_complete(crud.user.get_by_email(data["email"]))
    event_loop.run_until_complete(created.delete())


def test_create_user_open_already_created(
    client: TestClient, event_loop: EventLoop, normal_user_token_headers: Dict[str, str]
) -> None:
    data = {"email": default_user.email, "password": default_user.password}

    r = client.post(
        f"{settings.API_STR}/users/open", headers=normal_user_token_headers, json=data
    )

    assert r.status_code == 400



from datetime import timedelta

from fastapi import APIRouter, Depends, HTTPException
from fastapi.security import OAuth2PasswordRequestForm

from app import crud, models, schemas
from app.api import deps
from app.core import security
from app.core.config import settings


router = APIRouter()


@router.post("/access-token", response_model=schemas.Token)
async def login_access_token(form_data: OAuth2PasswordRequestForm = Depends()):
    """
    OAuth2 compatible token login, get an access token for future requests
    """

    user = await crud.user.authenticate(
        email=form_data.username, password=form_data.password
    )
    if not user:
        raise HTTPException(status_code=400, detail="Incorrect email or password")
    elif not crud.user.is_active(user):
        raise HTTPException(status_code=400, detail="Inactive user")
    access_token_expires = timedelta(minutes=settings.ACCESS_TOKEN_EXPIRE_MINUTES)
    return {
        "access_token": security.create_access_token(
            user.id, expires_delta=access_token_expires
        ),
        "token_type": "bearer",
    }


@router.post("/test-token", response_model=schemas.UserPydantic)
async def test_token(current_user: models.User = Depends(deps.get_current_user)):
    """
    Test access token
    """
    return await schemas.UserPydantic.from_tortoise_orm(current_user)



from fastapi import FastAPI
from starlette.middleware.cors import CORSMiddleware
from tortoise.contrib.fastapi import register_tortoise

from app.api.api import api_router
from app.core.config import settings


def create_app() -> FastAPI:
    app = FastAPI(
        title=settings.PROJECT_NAME,
        openapi_url=f"/openapi.json",
        docs_url="/",
    )
    # Set all CORS enabled origins
    if settings.BACKEND_CORS_ORIGINS:
        app.add_middleware(
            CORSMiddleware,
            allow_origins=[str(origin) for origin in settings.BACKEND_CORS_ORIGINS],
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )

    app.include_router(api_router, prefix=settings.API_STR)
    return app


app = create_app()

register_tortoise(
    app=app,
    db_url=settings.TORTOISE_DATABASE_URI,
    modules={"models": ["app.models"]},
    add_exception_handlers=True,
)


from typing import Dict
from fastapi.testclient import TestClient
from app.tests.conftest import default_user, default_superuser
from asyncio import AbstractEventLoop as EventLoop
from app.core.config import settings


def test_get_access_token(
    client: TestClient, event_loop: EventLoop, normal_user_token_headers: Dict[str, str]
) -> None:
    login_data = {"username": default_user.email, "password": default_user.password}
    r = client.post(f"{settings.API_STR}/login/access-token", data=login_data)
    tokens = r.json()
    assert r.status_code == 200
    assert "access_token" in tokens
    assert tokens["access_token"]


def test_use_access_token(
    client: TestClient, superuser_token_headers: Dict[str, str]
) -> None:
    r = client.post(
        f"{settings.API_STR}/login/test-token",
        headers=superuser_token_headers,
    )
    result = r.json()
    assert r.status_code == 200
    assert "email" in result


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastapi_plan',
 'fastapi_plan.template.hooks',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.api',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.api.routers',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.core',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.crud',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.models',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.schemas',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.tests',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.tests.api',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.tests.crud',
 'fastapi_plan.template.{{cookiecutter.project_name}}.app.tests.utils']

package_data = \
{'': ['*'],
 'fastapi_plan': ['template/*',
                  'template/{{cookiecutter.project_name}}/*',
                  'template/{{cookiecutter.project_name}}/config/*'],
 'fastapi_plan.template.{{cookiecutter.project_name}}.app': ['migrations/models/*']}

install_requires = \
['cookiecutter>=1.7.2,<2.0.0']

entry_points = \
{'console_scripts': ['fastapi-plan = fastapi_plan:main']}

setup_kwargs = {
    'name': 'fastapi-plan',
    'version': '0.3.1',
    'description': 'Dead simple template manager for FastAPI applications',
    'long_description': '[![GitHub license](https://img.shields.io/github/license/rafsaf/fastapi-plan)](https://github.com/rafsaf/fastapi-plan/blob/master/LICENSE)\n![PyPI](https://img.shields.io/pypi/v/fastapi-plan)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/fastapi-plan)\n![tests](https://github.com/rafsaf/fastapi-plan/actions/workflows/tests.yml/badge.svg)\n\ndead simple but powerful template manager for FastAPI applications.\n\n- [About](#about)\n- [Quickstart](#quickstart)\n  - [Initialization](#0-initialization)\n  - [Development](#1-development)\n  - [Debug (http)](#2-debug-http)\n  - [Production (https)](#3-production-https-own-domain)\n- [Project structure](#project-structure)\n- [High level overview](#high-level-overview)\n- [How to add new endpoint (step by step)](#how-to-add-new-endpoint)\n\n## About\n\nfeatures:\n- postgresql database with [Tortoise ORM](https://tortoise-orm.readthedocs.io/en/latest/index.html) as ORM\n- well organised, rock solid project structure (see section [Project structure](#project-structure))\n- ready-to-use user model, authentiaction system (JWT), hashing with Bcrypt\n- easy to undarstand config.py with settings (there is only one file for changes: `.env`)\n- out-of-the-box well-tested routes for login and user (register, read, read_me, update etc.)\n- aerich for migrations\n- well-designed tests folder filled with tests for existing user model/user endpoints\n- auto-generated strong passwords for database, secret_key and superuser password\n- poetry or pip\n- deployment ready docker-compose.prod.yml file with poetry, you will only need own domain\n\nfurthermore:\n- full [project structure schema](#project-structure)\n- [high level overview](#high-level-overview) how this project is organised and why, questions like where do the settings live or what every variable in `.env` file \n- step by step explanation [how to add new endpoint](#how-to-add-new-endpoint), from creating new model, adding schemas and routes to migrating database and writting tests (it\'s always better to have it and optionally adopt it, than wasting time trying to figure out the best dev path)\n\n\n## Quickstart\n\nNOTE: you will need [docker](https://www.docker.com/get-started) and optional but recommended [poetry](https://python-poetry.org/docs/) installed!\n\ninstall via pip (or poetry) **globally**:\n\n```bash\npip install fastapi-plan\n```\n\nthere are 3 docker-compose files available, one for development, one for running project via http, and the last one is for production with enabled https using traefic as a proxy (and letsencrypt for ssl), steps to initialize new project are the same for every approach, you can then choose from 1-3.\n\n### 0. INITIALIZATION\n\ninitialize new FastAPI project:\n\n```bash\nfastapi-plan\n```\n\nenter project_name and other information and after project is ready, `cd project_name` and continue installing dependencies:\n\n```bash\npoetry install\n\n# optional if you selected "requirements.txt" (with venv installed)\npip install -r requirements.txt\n```\n\n### 1. DEVELOPMENT\n\n\nsince we wanna use uvicorn in development, create only postgres container using docker-compose.yml file like that:\n\n```bash\ndocker-compose up -d\n```\n\nnow run aerich migrations and configure tortoise (and add first superuser)\n\n```bash\naerich upgrade\npython app/initial_data.py\n```\n\nfinally you can run this command to start uvicorn server\n\n```bash\nuvicorn app.main:app --reload\n```\n\n### 2. DEBUG (http)\n\nTo make it available from http://localhost on your local machine or http://your-host-name on VM just run\n\n```bash\ndocker-compose -f docker-compose.debug.yml up -d\n```\n\nThe diffrence between development approach is that web server automatically runs aerich and initial_data.py using shell script (`app/initial.sh`), so you don\'t have to do anything except changing some lines in `.env` file:\n\n1. `PROJECT_NAME` - it will show up in docs view as a name of project.\n2. `FIRST_SUPER_USER_EMAIL` - first account email\n3. `DEBUG` - when it\'s false, the `POSTGRES_SERVER` is set to `localhost` for development, so change it to `DEBUG=true` to use `db` postgres server.\n\n  \n### 3. PRODUCTION (https, own domain)\n\nTo make it available from https://your_domain.com on VM run\n\n```bash\ndocker-compose -f docker-compose.prod.yml up -d\n```\n\nThe diffrence between development approach is that web server automatically runs aerich and initial_data.py using shell script (`app/initial.sh`), so you don\'t have to do anything except changing some lines in `.env` file:\n\n1. `PROJECT_NAME` - it will show up in docs view as a name of project.\n2. `FIRST_SUPER_USER_EMAIL` - first account email\n3. `DEBUG` - when it\'s false, the `POSTGRES_SERVER` is set to `localhost` for development, so change it to `DEBUG=true` to use `db` postgres server.\n4. `DEFAULT_FROM_EMAIL` - your private email for ssl purposes, e.g. they will inform you shortly after some problems with you certificate.\n5. `MAIN_DOMAIN` - your own domain e.g. `example.com`\n\nPlesae also note that to get no-test certificate, you should comment line `"--certificatesresolvers.myresolver.acme.caserver=https://acme-staging-v02.api.letsencrypt.org/directory"` in `docker-compose.prod.yml` file, by default you will use test certifactes (to be sure that everything works, there are some hard limits on number of certifiactes you can ask per week!). You should comment line `"--log.level=DEBUG"` also (but it can be useful when debugging traefik). There would probably be problems anyway, just be sure that everything works via http using **2. DEBUG** apropach. If it then doesn\'t with the https, you should refer to traefik docs.\n\n## Project structure\n\n```\n|── app\n|    ├── api                              # endpoints/dependecies\n|    |\n|    ├── core                             # settings and security algorithms\n|    |\n|    ├── crud                             # CRUD operations\n|    |\n|    ├── migrations                       # for aerich migrations\n|    |\n|    ├── models                           # tortoise models\n|    |\n|    ├── schemas                          # pandatic schemas\n|    |\n|    ├── tests                            # tests\n|    |\n|    ├── initial.sh                       # initial shell script used by docker\n|    ├── initial_data.py                  # init database and add first superuser\n|    ├── main.py                          # main fastapi application file\n|\n├── config                                # nginx server config file\n|\n├── .env                                  # .env file with settings\n|\n├── Dockerfile                            # dockerfile for web app\n|\n├── aerich.ini                            # aerich (migrations) configuration\n|\n├── docker-compose.prod.yml               # puts it all together in prod (https)\n|\n├── docker-compose.debug.yml              # puts it all together in debug (http)\n|\n├── docker-compose.yml                    # puts it all together (development)\n|\n├── (optional) pyproject.toml             # python dependencies (poetry)\n|\n├── (optional) poetry.lock                # python dependencies (poetry)\n|\n├── requirements.txt                      # python dependencies (pip)\n```\n\n## High level overview\n\nThis project strucutre is mostly based on the [official template](#https://github.com/tiangolo/full-stack-fastapi-postgresql) (but not only) which is really great but unfortunatly does not support Tortoise ORM and is... (too?) complicated. All the security or problematic stuff (`app/core/security.py` with `verify_password` function, login and token routes, JWT token schemas) are just copied from there, so you can be preety sure it will work as expected.\n\nThe main thougts are:\n\n- There two sorts of settings, first one located in `.env` file for the ENTIRE project, and python-specific settings which lives in `app/core/config.py`, the file is based on pydantic solution (using dotenv lib). Why? Well, that\'s simple, this is due to [12factor methodology](https://12factor.net/), python-specific settings inherit from `.env` file, so this is the only place where you actually change something. If you have any problems understanding mentioned `config.py` file, just refer to [pydantic - settings management](https://pydantic-docs.helpmanual.io/usage/settings/), it\'s preety clear.\n\n- Models, crud, schemas, api routes, tests... it might be confusing how to actually ADD SOMETHING NEW here, but after following next section (learn by doing, step by step), it should be pretty easy\n\n- Database-related stuff is very convinient, taken mostly from [Tortoise ORM](https://tortoise-orm.readthedocs.io/en/latest/index.html) docs and just *working*. There is `register_tortoise` function in `main.py`, `TORTOISE_ORM` variable in `app/core/config.py`. Please, be aware that if you don\'t run `initial_data.py` SOMEHOW (in development- you have to do it yourself, in debug/production it is handled by shell script `initial.sh`, which also runs tests and migrations), you won\'t be able to connect to database. `initial_data.py` is hearbly based on the same named file in **official template** mentioned earlier. It has two responsibilities, first is running `init` function from Tortoise to initialize connection, and the second - creating first superuser (defined in `.env`) if one doesn\'t yet exists.\n\n- Migrations are also provided by Tortiose (the tool is aerich), docs can be found [here in aerich repo](https://github.com/tortoise/aerich). The default migration (default user model) file is already included. After changes in models (e.g. new model `Cars`), just run `aerich migrate`, `aerich upgrade` and you are good to go.\n\n- All tests lives in `tests` folder, with some pytest-specific content included. If you feel unconfortable with pytest, feel free to read articles about using it, and if you just want to see how to test new enpoints/models, just read next section.\n\n## How to add new endpoint\n\nLet\'s imagine we need to create API for a website where users brag about their dogs... or whatever, they just can crud dogs in user panel for some reason. We will add dummy model `Dog` to our API, with relation to the default table `User` and crud auth endpoints, then test it shortly.\n\n1. Create file `dog.py` in `app/models` folder:\n\n```python\nfrom tortoise import fields\nfrom tortoise.models import Model\n\n\nclass Dog(Model):\n    name = fields.CharField(max_length=100)\n    age = fields.IntField(null=True, default=None)\n    breed = fields.CharField(max_length=100, null=True, default=None)\n    owner = fields.ForeignKeyField("models.User", related_name="dogs")\n```\n\n2. Add import in `app/models.__init__.py`:\n\n```python\nfrom .dog import Dog # type: ignore\n```\n\n3. Migrate changes\n\n```bash\naerich migrate\naerich upgrade\n```\n\n4. Create file `dog.py` in `app/schemas` folder (pydantic schemas with typing support):\n\n```python\nfrom typing import Optional\nfrom tortoise import Tortoise\nfrom tortoise.contrib.pydantic.creator import (\n    pydantic_model_creator,\n    pydantic_queryset_creator,\n)\nfrom pydantic import BaseModel\nfrom app.models import Dog\n\n\n# Pydantic models from Tortoise models, pls refer\n# https://tortoise-orm.readthedocs.io/en/latest/examples/pydantic.html#basic-pydantic\n\nTortoise.init_models(["app.models"], "models")\nDogPydantic = pydantic_model_creator(Dog, exclude=("owner",))\nDogPydanticList = pydantic_queryset_creator(Dog, exclude=("owner",))\n\n# Unfortunately, it doesn\'t work the other way around\n\n\nclass DogCreate(BaseModel):\n    name: str\n    age: Optional[int]\n    breed: Optional[str]\n\n\nclass DogUpdate(BaseModel):\n    name: Optional[str]\n    age: Optional[int]\n    breed: Optional[str]\n\n```\n\n5. Add import in `app/schemas.__init__.py`:\n\n```python\nfrom .dog import DogUpdate, DogCreate, DogPydantic, DogPydanticList # type: ignore\n```\n\n6. Create `crud_dog.py` in `app/crud` folder\n\n```python\nfrom app.schemas import DogCreate, DogUpdate\nfrom app.crud.base import CRUDBase\nfrom app.models import Dog, User\n\n\nclass CRUDDog(CRUDBase[Dog, DogCreate, DogUpdate]):\n    def get_dogs_by_user(self, user: User, skip: int = 0, limit: int = 100):\n        return Dog.filter(owner=user).offset(offset=skip).limit(limit=limit)\n\n    async def create_dog_me(self, dog_in: DogCreate, user: User):\n        new_dog = await Dog.create(\n            name=dog_in.name, age=dog_in.age, breed=dog_in.breed, owner=user\n        )\n        return new_dog\n\n    async def get_by_id_and_user(self, dog_id: int, user: User):\n        return await Dog.get(id=dog_id, owner=user)\n\n    async def remove_all_user_dogs(self, user: User):\n        await Dog.filter(owner=user).delete()\n        return\n\n\ndog = CRUDDog(Dog)\n\n```\n\n7. Add import in `app/crud.__init__.py`:\n\n```python\nfrom .crud_dog import dog # type: ignore\n```\n\n\n8. Create `dogs.py` with endpoints in `app/api/routers` folder\n\n```python\nfrom fastapi import APIRouter, Depends, HTTPException, status\nfrom app import crud, models, schemas\nfrom app.api import deps\nfrom app.models import Dog\n\nrouter = APIRouter()\n\n\n@router.get("/{dog_id}", response_model=schemas.DogPydantic)\nasync def read_dog(\n    dog_id: int,\n):\n    dog = await crud.dog.get(dog_id)\n    if not dog:\n        raise HTTPException(\n            status_code=status.HTTP_404_NOT_FOUND,\n            detail="The dog does not exist in the system",\n        )\n    return await schemas.DogPydantic.from_tortoise_orm(dog)\n\n\n@router.post(\n    "/", response_model=schemas.DogPydantic, status_code=status.HTTP_201_CREATED\n)\nasync def create_dog_me(\n    dog_in: schemas.DogCreate,\n    current_user: models.User = Depends(deps.get_current_active_user),\n):\n    dog: Dog = await crud.dog.create_dog_me(dog_in, current_user)\n    return await schemas.DogPydantic.from_tortoise_orm(dog)\n\n\n@router.put("/", response_model=schemas.DogPydantic)\nasync def update_dog_me(\n    dog_id: int,\n    dog_in: schemas.DogUpdate,\n    current_user: models.User = Depends(deps.get_current_active_user),\n):\n    dog = await crud.dog.get_by_id_and_user(dog_id, current_user)\n    if not dog:\n        raise HTTPException(\n            status_code=status.HTTP_404_NOT_FOUND,\n            detail="The dog does not exist in the system",\n        )\n    new_dog = await crud.dog.update(dog, dog_in)\n    return await schemas.DogPydantic.from_tortoise_orm(new_dog)\n\n\n@router.delete("/", response_model=None, status_code=status.HTTP_204_NO_CONTENT)\nasync def delete_dog_me(\n    dog_id: int,\n    current_user: models.User = Depends(deps.get_current_active_user),\n):\n    dog = await crud.dog.get_by_id_and_user(dog_id, current_user)\n    if not dog:\n        raise HTTPException(\n            status_code=status.HTTP_404_NOT_FOUND,\n            detail="The dog does not exist in the system",\n        )\n    await crud.dog.remove(dog_id)\n    return None\n\n\n@router.delete("/all", response_model=None, status_code=status.HTTP_204_NO_CONTENT)\nasync def delete_dogs_me(\n    current_user: models.User = Depends(deps.get_current_active_user),\n):\n    await crud.dog.remove_all_user_dogs(current_user)\n    return None\n\n\n@router.get("/all", response_model=schemas.DogPydanticList)\nasync def read_all_dogs_me(\n    skip: int = 0,\n    limit: int = 100,\n    current_user: models.User = Depends(deps.get_current_active_user),\n):\n    dogs = crud.dog.get_dogs_by_user(current_user, skip, limit)\n    return await schemas.DogPydanticList.from_queryset(dogs)\n\n\n@router.get("/all/{user_id}", response_model=schemas.DogPydanticList)\nasync def read_all_dogs(\n    user_id: int,\n    skip: int = 0,\n    limit: int = 100,\n):\n    user = await crud.user.get(user_id)\n    if not user:\n        raise HTTPException(\n            status_code=status.HTTP_404_NOT_FOUND,\n            detail="The user does not exist",\n        )\n    dogs = crud.dog.get_dogs_by_user(user, skip, limit)\n\n    return await schemas.DogPydanticList.from_queryset(dogs)\n\n```\n\n9. Finally add those enpoints to the app with label "dogs", add this line in `app/api/api.py` file:\n\n```python\napi_router.include_router(dogs.router, prefix="/dogs", tags=["dogs"])\n```\n\n10. API endpoints are ready to go, you can play with them at `localhost:8000` by default\n\n11. Now we gonna create tests for crud and endpoints, let\'s first create utils for dog model (we can use it in multiple places in tests then), add `dog.py` in `app/tests/utils` folder:\n\n```python\nfrom asyncio import AbstractEventLoop as EventLoop\nfrom app import models\nimport app.tests.utils.utils as utils\n\n\ndef create_random_dog(user: models.User, event_loop: EventLoop) -> models.Dog:\n    name = utils.random_lower_string()\n    breed = utils.random_lower_string()\n    age = utils.random_integer_below_100()\n    dog: models.Dog = event_loop.run_until_complete(\n        models.Dog.create(name=name, breed=breed, age=age, owner=user)\n    )\n    return dog\n```\n\n12. Then add `test_dog.py` in `app/tests/crud` folder:\n\n```python\nimport pytest\nfrom asyncio import AbstractEventLoop as EventLoop\nfrom typing import List\nfrom app import crud, models, schemas\nfrom app.tests.utils.utils import (\n    random_lower_string,\n    random_integer_below_100,\n)\nfrom app.tests.utils.dog import create_random_dog\n\n\n@pytest.fixture(autouse=True)\ndef drop_dogs(event_loop: EventLoop) -> None:\n    yield\n    event_loop.run_until_complete(models.Dog.all().delete())\n\n\ndef test_get_dogs_by_user(event_loop: EventLoop, normal_user: models.User):\n\n    dog0 = create_random_dog(normal_user, event_loop)\n    dog1 = create_random_dog(normal_user, event_loop)\n    dog_lst: List[models.Dog] = list(\n        event_loop.run_until_complete(crud.dog.get_dogs_by_user(normal_user))\n    )\n\n    assert len(dog_lst) == 2\n    assert dog_lst[0].name == dog0.name\n    assert dog_lst[1].name == dog1.name\n    assert dog_lst[0].age == dog0.age\n    assert dog_lst[1].age == dog1.age\n    assert dog_lst[0].breed == dog0.breed\n    assert dog_lst[1].breed == dog1.breed\n\n\ndef test_create_dog_me(event_loop: EventLoop, normal_user: models.User):\n    name = random_lower_string()\n    breed = random_lower_string()\n    age = random_integer_below_100()\n    dog_in = schemas.DogCreate(name=name, breed=breed, age=age)\n    dog: models.Dog = event_loop.run_until_complete(\n        crud.dog.create_dog_me(dog_in, normal_user)\n    )\n\n    assert dog.name == name\n    assert dog.breed == breed\n    assert dog.age == age\n    assert dog.owner == normal_user\n\n\ndef test_get_dog_by_user(event_loop: EventLoop, normal_user: models.User):\n\n    dog0 = create_random_dog(normal_user, event_loop)\n\n    dog: models.Dog = event_loop.run_until_complete(\n        crud.dog.get_by_id_and_user(dog0.pk, normal_user)\n    )\n    assert dog == dog0\n\n\ndef test_remove_all_user_dogs(event_loop: EventLoop, normal_user: models.User):\n\n    create_random_dog(normal_user, event_loop)\n    create_random_dog(normal_user, event_loop)\n    dog_number0: int = event_loop.run_until_complete(\n        models.Dog.filter(owner=normal_user).count()\n    )\n    assert dog_number0 == 2\n    event_loop.run_until_complete(crud.dog.remove_all_user_dogs(normal_user))\n    dog_number1: int = event_loop.run_until_complete(\n        models.Dog.filter(owner=normal_user).count()\n    )\n    assert dog_number1 == 0\n```\n\n13. And then `test_dogs.py` for endpoints in `app/tests/api` folder\n\n',
    'author': 'rafsaf',
    'author_email': 'rafal.safin12@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rafsaf/fastapi-template',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import os

PROJECT_DIRECTORY = os.path.realpath(os.path.curdir)


def remove_file(filepath):
    os.remove(os.path.join(PROJECT_DIRECTORY, filepath))


if __name__ == "__main__":

    if "{{ cookiecutter.preffered_requirements_tool }}" == "poetry":
        pass
    else:
        remove_file("pyproject.toml")
        remove_file("poetry.lock")
